function integrateReduced(obj, k, tspan, x0, odeopt)
if nargin < 5
    odeopt = [];
end
matDot = @(matrix, vector) reshape(matrix(:).*repmat(vector,size(matrix,2),1), size(matrix));

obj.kVal = k;

odeopt = odeset(odeopt, 'Events', @(t,x) event(obj,t,x));
[t, x, dx] = obj.simulateReduced(k, tspan, x0, odeopt);

obj.tVal = t;

obj.theta = x(:,1);
obj.dtheta = x(:,2);
obj.d2theta = dx(:,2);

Phi = obj.PhiVec(k,obj.theta);
dPhi = obj.dPhiVec(k,obj.theta);
d2Phi = obj.d2PhiVec(k,obj.theta);

obj.qVal = Phi;
obj.dqVal = matDot(dPhi, obj.theta);
obj.d2qVal = matDot(d2Phi, obj.dtheta.*obj.dtheta) + matDot(dPhi, obj.d2theta);

obj.tauVal = obj.torque(obj.qVal, obj.dqVal, obj.d2qVal);

end

function [value, isterminal, direction] = event(obj,t,x)
    % Ends integration of the distance between the arm and bar is less than
    % delta
    value = 1;
    isterminal = 1;
    direction = 0;
    
    bar = [0.5, 0, 0]'; % Location of bar (streches in the y direction)
    
    delta = 0.05;
    gripPos = obj.gripArmPos(obj.PhiVec(obj.kVal, x(1)));
    if norm(gripPos([1,3]) - bar([1,3])) < delta
        value = 0;
    end
end

    
