function gorilla = gorilla_6dof

% Generalized coordinates
N = 6;
q = sym('q', [N,1]);
q = sym(q, 'real');
dq = sym('dq', [N,1]);
dq = sym(dq, 'real');
d2q = sym('d2q', [N,1]);
d2q = sym(d2q, 'real');

% Load robot parameters
parameters

g = [0,0,9.81]';
gorilla = Robot('Gorilla Robot', DH(0,0,0,sym(pi/2)), q, dq, d2q, g);   % Create an instance of Robot

% ======== Configuration ========
arms = gorilla.addLimb('Arms', gorilla.T0);
arms.addLink(q(1)-sym(pi/2), 0, l(8), 0);
arms.addLink(q(2), 0, l(7), 0);
arms.addLink(0, -w(3), 0, 0);
arms.addLink(q(3), 0, l(7), 0);
arms.addLink(q(4), 0, l(8), 0);

lb = l(4)+l(5)+l(6);
ll = l(2)+l(3);
body = gorilla.addLimb('Body', arms.links(2).T0*DH(0,-w(3)/2,0,0));
body.addLink(q(5), 0, lb, 0);
body.addLink(q(6), 0, ll, 0);

arms.links(1).addMass(m4, [-l(8)/2,0,0]', diag(I4));
arms.links(1).addMass(me, [0,0,0]', diag(Ie));
arms.links(2).addMass(m3, [-l(7)/2,0,0]', diag(I3));
arms.links(2).addMass(ms, [0,0,0]', diag(Is));
arms.links(4).addMass(ms, [-l(7),0,0]', diag(Is));
arms.links(4).addMass(m3, [-l(7)/2,0,0]', diag(I3));
arms.links(4).addMass(me, [0,0,0]', diag(Ie));
arms.links(5).addMass(m4, [-l(8)/2,0,0]', diag(I4));

body.links(1).addMass(mb, [-lb+lb2,0,0]', diag(Ib));
body.links(1).addMass(mh, [-l(4),0,0]', diag(Ih));
body.links(2).addMass(2*m2, [-l(2)-l(3)/2,0,0]', diag(2*I2));
body.links(2).addMass(2*mk, [-l(2),0,0]', diag(2*Ik));
body.links(2).addMass(2*m1, [-l(2)/2,0,0]', diag(2*I1));
body.links(2).addMass(2*ma, [0,0,0]', diag(2*Ia));

gorilla.calculateCoM();

end 