% clear all
close all

load('sim.mat');        % Previous simulation data
load('ideal.mat');      % Ideal trajectories
load('opt.mat');        % Scalar controller coefficients


q_a = q_sim.signals.values';
dq_a = dq_sim.signals.values';

theta_a = q_a(1,:);
dtheta_a = dq_a(1,:);

q_des = PhiFun(theta_a);
dq_des = dPhiFun(theta_a).*repmat(dtheta_a,size(q_a,1),1);

figure(6);
subplot(2,1,1)
plot(q_sim.time,q_a(2,:),q_sim.time,q_des(2,:));
legend('q_2 (measured)','q_2 (desired)');
ylabel('Angle [rad]');
set(gca,'XLim',[0,q_sim.time(end)]);
subplot(2,1,2)
plot(dq_sim.time,dq_a(2,:),dq_sim.time,dq_des(2,:));
set(gca,'XLim',[0,q_sim.time(end)]);
legend('dq_2 (measured)','dq_2 (desired)');
xlabel('Time [s]');
ylabel('Angular Velocity [rad/s]');

figure(7)
subplot(1,2,1)
% plot(q_a(1,:),dq_a(1,:),q_des(1,:),dq_des(1,:),q_star(1,:),dq_star(1,:));
plot(q_a(1,:),dq_a(1,:),q_star(1,:),dq_star(1,:),q_a(1,1),dq_a(1,1),'o');
legend('Actual orbit','Desired orbit','Initial position');
title('Phase Portrait for \theta(t)')
xlabel('\theta')
ylabel('d\theta/dt')
subplot(1,2,2)
% plot(q_a(2,:),dq_a(2,:),q_des(2,:),dq_des(2,:),q_star(2,:),dq_star(2,:));
plot(q_a(2,:),dq_a(2,:),q_star(2,:),dq_star(2,:),q_a(2,1),dq_a(2,1),'o');
legend('Actual orbit','Desired orbit','Initial position');
title('Phase Portrait for q_2(t)')
xlabel('q_2')
ylabel('dq_2/dt')

figure(1);
plot(q_sim.signals.values(:,1),dq_sim.signals.values(:,1));
hold all
plot(q_star(1,:),dq_star(1,:));

plot(q_sim.signals.values(:,2),dq_sim.signals.values(:,2));
plot(q_star(2,:),dq_star(2,:));
plot(q_sim.signals.values(1,1),dq_sim.signals.values(1,1),'o');   % Starting point
plot(q_sim.signals.values(1,2),dq_sim.signals.values(1,2),'o');

title('Phase plots')
legend('(x, dx)','(x*, dx*)','(\theta, d\theta)', '(\theta*, d\theta*)','(x_0,dx_0)', '(\theta_0, d\theta_0)');
xlabel('x/theta');
ylabel('dx/dtheta');
hold off


figure(2);
plot(x_perp_sim.time,squeeze(x_perp_sim.signals.values))

title('x_{perp}');
legend('I','y','dy');
xlabel('time [s]');


figure(3);
subplot(2,1,1);
plot(q_sim.time,q_sim.signals.values');
% xlabel('time [s]');
legend('x', 'theta');
title('states');

subplot(2,1,2);
plot(dq_sim.time,squeeze(dq_sim.signals.values));
xlabel('time [s]');
legend('dx', 'dtheta');


figure(4);
plot(tau_sim.time,squeeze(tau_sim.signals.values));
xlabel('time [s]');
ylabel('\tau');
% axis(t_star([1,end,1,end]));


figure(5);
subplot(3,1,1);
plot(u_sim.time,squeeze(u_sim.signals.values));
title('System Control');
ylabel('u(t)');

subplot(3,1,2);
plot(v_sim.time,squeeze(v_sim.signals.values));
title('Auxiliary Control');
ylabel('v(t)');

subplot(3,1,3);
Kt = nan(3,length(tau_sim.time));
for i=1:length(tau_sim.time);
    Kt(:,i) = bezier(tau_sim.signals.values(i),Pmin,t_star(1),t_star(end));
end
plot(tau_sim.time,Kt)
title('Control Gains');
ylabel('K(t)');
xlabel('Time [s]');
legend('k_1','k_2','k_3');


