function M = animate(t, qMat, fps)
    if length(t) < fps*t(end)
        disp(['Warning: not enough frames for smooth playback with ', num2str(fps), ' fps']);
    end

    currentframe = 0;
    for i=1:length(t)
        if t(i) > currentframe/fps
            currentframe = currentframe + 1;
            q = qMat(i,:)';
            pendulumPlot(q(1),q(2));
            %hold on
            M(currentframe) = getframe;
        end
    end
    hold off
end

function pendulumPlot(x, theta, l)
    if nargin < 3
        l = 1;
    end
    
    base = [x;0];
    pendulumEnd = [x - l*sin(theta), l*cos(theta)];
    
    w = 1;      % Cart width
    h = 0.3;    % Cart height
    r = 0.1;    % Wheel/mass radius
    
    ylim = [0,3*r+h+l];
    
    s = 0:0.001:2*pi;
    wheel_rear.x = x - w/4 + r*cos(s);
    wheel_rear.y = r*(1+sin(s));
    wheel_front.x = wheel_rear.x+w/2;
    wheel_front.y = wheel_rear.y;
    cart.x = x+w/2*[-1,-1,1,1,-1];
    cart.y = 2*r + [0,h,h,0,0];
    rod.x = x+[0,-l*sin(theta)];
    rod.y = 2*r + h + [0,l*cos(theta)];
    mass.x = rod.x(end)+r*cos(s);
    mass.y = rod.y(end)+r*sin(s);
    
    x_coords = [wheel_rear.x,nan,wheel_front.x,nan,cart.x,nan,rod.x,nan,mass.x];
    y_coords = [wheel_rear.y,nan,wheel_front.y,nan,cart.y,nan,rod.y,nan,mass.y];
    
    plot(x_coords,y_coords);
%     plot([base(1),pendulumEnd(1)], [base(2),pendulumEnd(2)]);
    axis([-2,2,ylim]);
    set(gca,'DataAspectRatio',[1,1,1])
    
end