function B = bezier_sym(t, P, t0, tend)
% bezier_sym
%   B = bezier_sym(t,P,t0,tend) computes the symbolic bezier polynomial for
%   the variable t. The domain of t is [t0,tend] and can be both symbolic,
%   both numeric, or a combination.
%   If P is a m-by-n matrix, B is a column vector of length m with element
%   k containing the bezier polynomials given the k'th row of P.

if ~isa(t,'sym')
    error('input must be a symbolic variable');
end
if ~isscalar(t)
    error('input must be scalar');
end

% Calculations
m = size(P,1);                  % Number of independent Bzier curves to be created
n = size(P,2)-1;              	% Order of the Bzier curve
t = (t-t0)/(tend-t0);           % Remap time to [0,1]

B = sym(zeros(m,1));
for i=0:n
    B = B + bernstein_polynomial(i,n,t).*P(:,i+1);
end

end

function b = bernstein_polynomial(i,n,t)
b = nchoosek(n,i)*t^i*(1-t)^(n-i);
end
