function fun_compute_integral_terms

% Input (initialized elsewhere)
global theta_star dtheta_star                   % Desired trajectory
global theta_control_min theta_control_max      % Control range for which integral should be calculated
global theta_star_0                             % Initial condition
global dirs                                     % Directory struct

% Outputs (initialized here)
% theta_s Psi_precalc phi_star                  % Saved to dirs.data folder

load('model.mat', 'timeFun');
[theta_s, Psi1_s, Psi2_s] = fct_integral_terms(theta_control_min,theta_control_max,theta_star_0,1000);

% phi_star = atan2(dtheta_star,theta_star)';
phi_star = timeFun(theta_star,dtheta_star);


Psi_precalc = -Psi1_s*dtheta_star(1)+Psi2_s;
save([dirs.data, filesep, 'integral'], 'theta_s', 'Psi1_s', 'Psi2_s', 'phi_star', 'Psi_precalc');

end

function [theta_s, Psi1_s, Psi2_s] = fct_integral_terms(theta_start,theta_end,theta_0,samples)
% Compute terms of the integral as list for the interval [theta_start, theta]
% Matlab code: (c) Uwe Mettin, 2009 (based on Ians code)
% Modified 04.12 by Stian Askeland
%   Now computes the integral from theta_0 to each theta in the range
%   [theta_start,theta_end]

theta_s = linspace(theta_start,theta_end,samples)';

% tolerance
tol = 1e-6;

warning off all
% Psi1 (first term)
Psi1_s = calc_Psi1(theta_s,theta_0,tol);

% Psi2 (second term)
Psi2_s = calc_Psi2(theta_s,theta_0,tol,Psi1_s);
warning on all

end

%--------------------------

function y = calc_Psi1(theta_s,theta_0,tol)
theta_vec = [theta_0;theta_s];
y = cumquadl(@fct_calc_Psi1,theta_vec,tol);
y = y(2:end);   % Drop first element as integral now starts at theta_0
y = exp(-2*y);
end

function y = fct_calc_Psi1(theta)
% global alphaFun betaFun                % Scalar dynamics functions
y = betaFun(theta) ./ alphaFun(theta);
end

%--------------------------

function y = calc_Psi2(theta_s,theta_0,tol,Psi1_s)
% global alphaFun gammaFun                % Scalar dynamics functions
theta_vec = [theta_0;theta_s];
y = cumquadl(@fct_calc_Psi2,theta_vec,tol);
y = y(2:end);   % Drop first element as integral now starts at theta_0
y = 2*Psi1_s.*y;

    function z = fct_calc_Psi2(theta)
    Psi_theta = interp1(theta_s,Psi1_s,theta,'spline');
    z = gammaFun(theta) ./ alphaFun(theta) ./ Psi_theta;
    end

end

function y = cumquadl(fun,x,varargin)
%Cumulative integral of a function using QUADL
y = zeros(size(x));
for k = 2:numel(x)
    y(k) = y(k-1) + quadl(fun,x(k-1),x(k),varargin{:});
end

end