function fun_control_search_bezier
% Searches for bezier coefficients which minimizes the weighted norm of the
% state transition matrix of the system
%   dx = Ax + Bu
% where the dimensions are
%   A - n-by-n
%   B - n-by-m
%   x - n-by-1
%   u - m-by-1
% Inputs:
%   n - number of states
%   m - number of control inputs
%   p - number of control points for each bezier curve

global mincost, mincost = inf;
% global t_star
global Pmin

global dirs

% global n

load([dirs.data,filesep,'ideal']);
old_opt = load([dirs.data,filesep,'opt'],'Cmin','Pmin','Emin');

B_temp = BlinFunT(t_star(1));

N = size(B_temp,1);  % length of x_perp (should be 2n-1 when underactuated of degree 1
m = size(B_temp,2);  % number of control inputs
p = 4;  % number of control points for each bezier curve

C0 = 1*ones(N*m,p-2);  % Number of free control points equals p-2 (last two are set for smooth continuity)
% C0 = 100*rand(N*m,p-1);  % Number of free control points equals p-2 (last two are set for smooth continuity)
% C0 = old_opt.Cmin;
% C0(:,end+1) = C0(:,end);
% C0(:,end+1) = (C0(:,end)-C0(:,end-1))/2 + C0(:,1);

% options = optimset('TolFun', 0.1, 'TolX', 0.1, 'OutputFcn', @outputFun);
options = optimset('MaxFunEvals', 1e6, 'MaxIter', 1e6);
[Cmin, Emin] = fminsearch(@(C)controlCost(C,N,m), C0(:), options);
Cmin = reshape(Cmin,N*m,[]);

% Cmin = old_opt.Cmin;
% Emin = old_opt.Emin;

Pmin = Cmin;
Pmin(:,end+1) = 2*Pmin(:,1)-Pmin(:,2);
Pmin(:,end+1) = Pmin(:,1);

save([dirs.data, filesep, 'opt'], 'Cmin', 'Emin', 'Pmin');

t0 = t_star(1);
tend = t_star(end);

Atilde = @(t) AlinFunT(t) - BlinFunT(t)*controlLaw(t,Cmin,t0,tend,N,m);

[T,Psi] = ode45(@(t,Psi) dPsi(t,Psi,controlLaw(t,Cmin,t0,tend,N,m)), [t0,tend], eye(N));
[T2,Aint] = ode45(@(t,x) reshape(Atilde(t),N*N,1), [t0,tend], zeros(N*N,1));

Psi = reshape(Psi',N,N,[]);
Aint = reshape(Aint',N,N,[]);

PsiT = Psi(:,:,end);
AintT = Aint(:,:,end);
PsiT2 = expm(AintT);

end

function c = controlCost(C,n,m)
    
C = reshape(C,n*m,[]);

global t_star
global mincost

t0 = t_star(1);
tend = t_star(end);


% Find state transition matrix
[T,Psi] = ode45(@(t,Psi) dPsi(t,Psi,controlLaw(t,C,t0,tend,n,m)), [t0,tend], eye(n));
Psi3D = reshape(Psi',n,n,[]);

% State transition matrix at the end of a period
D = NaN(length(T),n);       % Matrix containing the eigenvalues at each timestep
normVec = NaN(length(T),1);   % Matrix containing the norm of Psi at each timestep
for i=1:length(T)
    PsiT = reshape(Psi(i,:),n,n);

    % Eigenvalues of state transition matrix at t=t_star(i)
    D(i,:) = abs(eig(PsiT));
    normVec(i) = norm(PsiT);
end
d = D(end,:); % Eigenvalues at t=T

weightVec = (exp(T)-exp(t0))./(exp(tend)-exp(t0));
% weightVec = T+0.5;
% weightVec = 0.5*(1-cos(pi*(1:length(T))/length(T)))';
% weightVec = [zeros(length(T)-1,1);1];
% weightVec = 1 - exp(-T);
% weightVec = ones(size(normVec));


% Cost
% R_cost = eye(7);
% R_cost(5,5)= 100; R_cost(6,6)=100; R_cost(7,7) = 100;
if isempty(lastwarn)
%     c = 100*e_T + norm(D);
%     c = max(d);
%     c = norm(d)                     % Cost is the maximum eigenvalue at the end of the period
%     c = norm(D);                 	% Cost is the norm of the eigenvalues throughout the period
%     c = norm(max(D,2)) + max(d); 	% Weighted sum of the above
% 
%     c = norm(PsiT);                 % The matrix norm of Psi at the end of the period
%     c = weightVec'*normVec;         % The weightet norm   
%     c = sum(normVec.^2);
    c = trapz(T, weightVec.*normVec);
    
else
    c = Inf;
    lastwarn('');
end

mincost = min(mincost, c);
P = C;
P(:,end+1) = 2*P(:,1)-P(:,2);
P(:,end+1) = P(:,1);
figure(1)
plot(T,bezier(T,P),linspace(T(1),T(end),size(P,2)),P')
drawnow;

disp('');
disp('Control Coefficients:')
disp(C);
disp('Eigenvalues at t=T');
disp(abs(d));
disp('Norm at t=T');
disp(normVec(end));
disp('Maximum eigenvalue at t=T');
disp(max(abs(d)));
disp('Cost:');
disp(c);
disp('Minimum cost:');
disp(mincost);

figure(2)
plot(T,normVec)
drawnow;
end

function K = controlLaw(t, C, t0, tend,n,m)

% Add 2 more columns to K such that the corresponding bezier curve is periodic and smooth
P = C;
P(:,end+1) = 2*P(:,1)-P(:,2);
P(:,end+1) = P(:,1);

K = bezier(t,P,t0,tend)';
K = reshape(K,m,n);
end

function dPsi = dPsi(t, Psi, K)

n = size(K,2);
Psi = reshape(Psi,n,n);             % Create state transition matrix            (vector -> matrix)

A = AlinFunT(t);
B = BlinFunT(t);

Afb = A - B*K;                      % Feedback system

dPsi = Afb*Psi;                   	% Derivative of state transition matrix     (matrix -> matrix)
dPsi = dPsi(:);                     % Vectorize                                 (matrix -> vector)

end
