function fun_define_input_gorilla

% Syms
global theta dtheta d2theta
global y dy v
global q dq d2q
global xi nu xi_star

global n                                    % Number of states

global Msym Csym Gsym Bsym Bperpsym         % System matrices
global Phi dPhi d2Phi                       % Syncronization functions
global alpha beta gamma

global t0 tmax                              % Start & maximum time
global theta_star_0 dtheta_star_0           % Initial values
global theta_control_min theta_control_max  % Range for which control integral should be calculated

global odeEventFun

global Linv N                               % Transformations: dq = L*[dtheta;dy], d2q = L*[d2theta;d2y]+N

global Smat                                 % Selector matrix. Rearranges q such that the free variable is the bottom element

global dirs                                 % Directory struct


% Syms
syms u real                                 % Control

% Initialize gorilla
gorilla = gorilla_6dof;

load('bezier_coefficients_gorilla.mat');    % Contains K (bezier coefficients), theta0 and thetaend

% Control parameters
% The integral term is calculated between these limits
theta_control_min = 1.5*theta0;
theta_control_max = 1.5*thetaend;

theta_star_0 = theta0;
dtheta_star_0 = 0;

t0 = 0;         % Initial time
tmax = 10;      % Maximum time allowed (simulation will break after one period)

timeFun = @(theta,dtheta) atan2(dtheta,theta);   % Function mapping from theta,dtheta to some monotonically increasing quantity

% odeEventFun = @(t,z) fun_ode_event_gorilla(t,z,thetaend);
odeEventFun = @fun_ode_event_pendulum;


save([dirs.data,filesep,'model'],'timeFun');

if exist('sym_mat.mat')
    load('sym_mat');
else
    % This part only has to run if the symbolic matrices have not yet been
    % saved
    
    B = [zeros(1,gorilla.N-1); eye(gorilla.N-1)];
    Bperp = [1,zeros(1,gorilla.N-1)];
    Bsym = sym(B);
    Bperpsym = sym(Bperp);

    % Generalized coordinates
    q = gorilla.q;
    dq = gorilla.dq;
    d2q = gorilla.d2q;

    n = length(q);

    Smat = [zeros(n-1,1),eye(n-1); 1, zeros(1,n-1)];

    theta = Bperp*q;
    dtheta = Bperp*dq;
    d2theta = Bperp*d2q;

    index.theta = 1;
    index.y = 2:n;

    save([dirs.data, filesep, 'indices'], 'index');

    y = sym('y',[n,1]);
    y = sym(y,'real');
    y = y(2:end);
    dy = sym('dy',[n,1]);
    dy = sym(dy,'real');
    dy = dy(2:end);
    v = sym('v',[n,1]);
    v = sym(v,'real');
    v = v(2:end);

    xi = [q;dq;d2q];
    nu = [0;y;0;dy;0;v];

    [Msym,Csym,Gsym] = gorilla.calculateDynamics;


    % Virtual Holonomic Constraints

    P = K;
    m = size(P,2)-1;
    dP = m/(thetaend-theta0)*diff(P,1,2);
    d2P = (m-1)/(thetaend-theta0)*diff(dP,1,2);

    Phi = bezier_sym(theta,P,theta0,thetaend);
    dPhi = bezier_sym(theta,dP,theta0,thetaend);           % Partial derivative
    d2Phi = bezier_sym(theta,d2P,theta0,thetaend);         % Partial derivative

    % alpha = Bperpsym*subs(Msym,q,Phi)*dPhi;
    % beta = Bperpsym*( Csym*dPhi + Msym*d2Phi );
    % beta = subs(beta,[q;dq],[Phi,dPhi]);
    % gamma = Bperpsym*subs(Gsym,q,Phi);

    alpha = subs(Bperpsym*Msym,q,Phi)*dPhi;
    beta = subs(Bperpsym*Msym,q,Phi)*d2Phi + subs(Bperpsym*Csym,[q;dq],[Phi;dPhi])*dPhi;
    gamma = subs(Bperpsym*Gsym, q, Phi);


    L = [dPhi, [zeros(1,n-1);eye(n-1)]];
    Linv = eye(size(L))/L;
    N = d2Phi*dtheta^2;

    xi_star = [Phi;dPhi*dtheta;dPhi*d2theta+d2Phi*dtheta^2];
    
    save([dirs.data,filesep,'sym_mat']);

    % Create functions
    % matlabFunction(sym(Minvsym),'vars',{q},'file',[dirs.generated_functions, filesep, 'MfunInv']);
    matlabFunction(sym(Msym),'vars',{q},'file',[dirs.generated_functions, filesep, 'Mfun']);
    matlabFunction(sym(Csym),'vars',{q,dq},'file',[dirs.generated_functions, filesep, 'Cfun']);
    matlabFunction(sym(Gsym),'vars',{q},'file',[dirs.generated_functions, filesep, 'Gfun']);

    matlabFunction(sym(Bsym),'vars',{q},'file',[dirs.generated_functions, filesep, 'Bfun']);
    matlabFunction(sym(Bperpsym),'vars',{q},'file',[dirs.generated_functions, filesep, 'Bperpfun']);

    matlabFunction(sym(Bperpsym*Msym),'vars',{q},'file',[dirs.generated_functions, filesep, 'BMfun']);
    matlabFunction(sym(Bperpsym*Csym),'vars',{q,dq},'file',[dirs.generated_functions, filesep, 'BCfun']);
    matlabFunction(sym(Bperpsym*Gsym),'vars',{q},'file',[dirs.generated_functions, filesep, 'BGfun']);

    matlabFunction(sym(Phi),'vars',{theta},'file',[dirs.generated_functions, filesep, 'PhiFun']);
    matlabFunction(sym(dPhi),'vars',{theta},'file',[dirs.generated_functions, filesep, 'dPhiFun']);
    matlabFunction(sym(d2Phi),'vars',{theta},'file',[dirs.generated_functions, filesep, 'd2PhiFun']);

    matlabFunction(sym(alpha),'vars',{theta},'file',[dirs.generated_functions, filesep, 'alphaFun']);
    matlabFunction(sym(beta),'vars',{theta},'file',[dirs.generated_functions, filesep, 'betaFun']);
    matlabFunction(sym(gamma),'vars',{theta},'file',[dirs.generated_functions, filesep, 'gammaFun']);

    matlabFunction(sym(Smat(1:end-1,:)'),'file',[dirs.generated_functions, filesep, 'SFun']); % The part of Smat corresponding to the actuated coordinates
    matlabFunction(sym(eye(size(L))/L),'vars',{theta,y},'file',[dirs.generated_functions, filesep, 'LinvFun']);
    matlabFunction(sym(N),'vars',{theta,dtheta,y,dy},'file',[dirs.generated_functions, filesep, 'NFun']);

end

end
