function fun_define_input

% Syms
global theta dtheta d2theta x dx d2x
global y dy v
global q dq d2q
global xi nu xi_star

global n                                    % Number of states

global Msym Csym Gsym Bsym Bperpsym      % System matrices
global Phi dPhi d2Phi                       % Syncronization functions
% global alphaFun betaFun gammaFun            % Scalar dynamics functions
global alpha beta gamma

global t0 tmax                              % Start & maximum time
global theta_star_0 dtheta_star_0           % Initial values
global theta_control_min theta_control_max  % Range for which control integral should be calculated

global Smat N

global odeEventFun

global dirs                                 % Directory struct


% Syms
syms theta dtheta d2theta x dx d2x real     % Generalized coordinates
syms y dy real                              % Error vector
syms u v real                               % Control
syms M m l g a l2 real                  	% Physical parameters


% Physical Parameters
M = 1;
m = 1;
l = 1;
g = 9.81;
a = 0;

l2 = 1.5;
% l2 = 0.5;


% Generalized coordinates
q = [x;theta];
dq = [dx;dtheta];
d2q = [d2x;d2theta];

xi = [q;dq;d2q];
nu = [y;0;dy;0;v;0];

n = length(q);

index.theta = 2;
index.y = 1:n-1;

save([dirs.data, filesep, 'indices'], 'index');

Smat = eye(n);  % Free coordinate already on the bottom

% System Matrices
Msym = [M+m,       	-m*l*cos(theta);
    -cos(theta),     l];
Csym = [0, m*l*dtheta*sin(theta);
    0,  0];
Gsym = [0; -g*sin(theta)];
Bsym = [1;0];

Bperpsym = null(Bsym')';
Minvsym = sym(eye(size(Msym)))/Msym;

% With error
delta = 1;
MsymActual = [M+m*delta,       	-m*delta*l*cos(theta);
    -cos(theta),     l];
CsymActual = [0, m*delta*l*dtheta*sin(theta);
    0,  0];
GsymActual = Gsym;
BsymActual = Bsym;


% Virtual Holonomic Constraints
Phi = [a + l2*sin(theta); theta];
dPhi = [l2*cos(theta);1];           % Partial derivative
d2Phi = [-l2*sin(theta);0];         % Partial derivative

xi_star = [Phi;dPhi*dtheta;dPhi*d2theta+d2Phi*dtheta^2];

alpha = simplify(subs(Bperpsym*Msym,q,Phi)*dPhi);
beta = simplify(subs(Bperpsym*Msym,q,Phi)*d2Phi + subs(Bperpsym*Csym,[q;dq],[Phi;dPhi])*dPhi);
gamma = simplify(subs(Bperpsym*Gsym, q, Phi));

odeEventFun = @fun_ode_event_pendulum;

L = [[eye(n-1);zeros(1,n-1)], dPhi];
% Linv = eye(size(L))/L;
N = d2Phi*dtheta^2;


% Test
% K = [1,0]/L/Msym*Bsym;
% R = [1,0]/L*(Msym\(Csym*dq+Gsym)+N);
% 
% fun = Bperpsym*(Msym*d2q + Csym*dq + Gsym);
% g = alpha*d2theta+beta*dtheta^2+gamma-subs(fun,xi,xi_star+nu);
% 
% d2theta_1 = simplify( subs(L\(Msym\(Bsym*u - Csym*dq - Gsym) - N) ,u, K\(v+R)));
% d2theta_2 = simplify(alpha\(g-beta*dtheta^2-gamma));
% 
% d2theta_1fun = matlabFunction(d2theta_1, 'vars', {theta,dtheta,d2theta,y,dy,v});
% d2theta_2fun = matlabFunction(d2theta_2, 'vars', {theta,dtheta,d2theta,y,dy,v});


theta_star_0 = -0.1;
dtheta_star_0 = 0;

t0 = 0;         % Initial time
tmax = 10;      % Maximum time allowed (simulation will break after one period)

timeFun = @(theta,dtheta) atan2(dtheta,theta)';     % Function mapping from theta,dtheta to some monotonically increasing quantity

save([dirs.data,filesep,'model'],'timeFun')

% Create functions
% matlabFunction(sym(Minvsym),'vars',{q},'file',[dirs.generated_functions, filesep, 'MfunInv']);
matlabFunction(sym(Msym),'vars',{q},'file',[dirs.generated_functions, filesep, 'Mfun']);
matlabFunction(sym(Csym),'vars',{q,dq},'file',[dirs.generated_functions, filesep, 'Cfun']);
matlabFunction(sym(Gsym),'vars',{q},'file',[dirs.generated_functions, filesep, 'Gfun']);

matlabFunction(sym(Bsym),'vars',{q},'file',[dirs.generated_functions, filesep, 'Bfun']);
matlabFunction(sym(Bperpsym),'vars',{q},'file',[dirs.generated_functions, filesep, 'Bperpfun']);

matlabFunction(sym(Bperpsym*Msym),'vars',{q},'file',[dirs.generated_functions, filesep, 'BMfun']);
matlabFunction(sym(Bperpsym*Csym),'vars',{q,dq},'file',[dirs.generated_functions, filesep, 'BCfun']);
matlabFunction(sym(Bperpsym*Gsym),'vars',{q},'file',[dirs.generated_functions, filesep, 'BGfun']);

% Note: these must be updated manually to allow for vector input
% matlabFunction(sym(Phi),'vars',{theta},'file',[dirs.generated_functions, filesep, 'PhiFun']);
% matlabFunction(sym(dPhi),'vars',{theta},'file',[dirs.generated_functions, filesep, 'dPhiFun']);
% matlabFunction(sym(d2Phi),'vars',{theta},'file',[dirs.generated_functions, filesep, 'd2PhiFun']);

matlabFunction(sym(alpha),'vars',{theta},'file',[dirs.generated_functions, filesep, 'alphaFun']);
matlabFunction(sym(beta),'vars',{theta},'file',[dirs.generated_functions, filesep, 'betaFun']);
matlabFunction(sym(gamma),'vars',{theta},'file',[dirs.generated_functions, filesep, 'gammaFun']);

matlabFunction(sym(Smat(:,1:end-1)),'file',[dirs.generated_functions, filesep, 'SFun']);
matlabFunction(sym(eye(size(L))/L),'vars',{theta,y},'file',[dirs.generated_functions, filesep, 'LinvFun']);
matlabFunction(sym(N),'vars',{theta,dtheta,y,dy},'file',[dirs.generated_functions, filesep, 'NFun']);

% Actual system functions
matlabFunction(sym(MsymActual),'vars',{q},'file',[dirs.generated_functions, filesep, 'MfunActual']);
matlabFunction(sym(CsymActual),'vars',{q,dq},'file',[dirs.generated_functions, filesep, 'CfunActual']);
matlabFunction(sym(GsymActual),'vars',{q},'file',[dirs.generated_functions, filesep, 'GfunActual']);
matlabFunction(sym(BsymActual),'vars',{q},'file',[dirs.generated_functions, filesep, 'BfunActual']);

% Control parameters
% The integral term is calculated between these limits
theta_control_min = -0.3;
theta_control_max = 0.3;

end
